package loadtests.examples.loadcoder;

import static com.loadcoder.statics.Statics.PER_SECOND;
import static com.loadcoder.statics.Statics.PER_THREAD;
import static com.loadcoder.statics.Statics.SECOND;
import static com.loadcoder.statics.Statics.SHARED;
import static com.loadcoder.statics.Statics.duration;

import com.loadcoder.load.LoadUtility;
import com.loadcoder.load.chart.logic.ResultChart;
import com.loadcoder.load.chart.logic.RuntimeChart;
import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenario;
import com.loadcoder.result.Result;

/**
 * This class shows different examples of how to run a Loadcoder test
 */
public class ExecutionExample {

	public static void main(String[] args) {
		new ExecutionExample().twoLoadsInOneExecution();
	}

	LoadScenario loadScenario1 = new LoadScenario() {

		@Override
		public void loadScenario() {

			load("sleep1", () -> {
				LoadUtility.randomSleep(10, 20);
			}).perform();
		}
	};

	LoadScenario loadScenario2 = new LoadScenario() {

		@Override
		public void loadScenario() {

			load("sleep2", () -> {
				LoadUtility.randomSleep(10, 20);
			}).peak(5, 0.1).perform();
		}
	};

	public void twoLoadsInOneExecution() {

		Load shortLoad = new LoadBuilder(loadScenario1).amountOfThreads(10).stopDecision(duration(2 * SECOND))

				.throttle(2, PER_SECOND, PER_THREAD).build();

		Load longerLoad = new LoadBuilder(loadScenario2).amountOfThreads(20).stopDecision(duration(4 * SECOND))
				.throttle(5, PER_SECOND, SHARED).build();

		/*
		 * Two Load objects are used in below execution
		 */
		RuntimeChart chart = new RuntimeChart();
		Result result = new ExecutionBuilder(shortLoad, longerLoad).storeAndConsumeResultRuntime(chart).build()
				.execute().andWait().getReportedResultFromResultFile();

		result.summaryStandard().build().prettyPrint();

		/*
		 * Below another execution is performed where the loadScenario1 is used again.
		 * The result is only stored in memory and once finished, a summary is printed
		 */
		Load oneIterationLoad = new LoadBuilder(loadScenario1).build();
		result = new ExecutionBuilder(oneIterationLoad).storeResultRuntime().build().execute().andWait()
				.getReportedResultFromResultFile();

		result.summaryStandard().build().prettyPrint();

		chart.waitUntilClosed();
	}

	public void onlyStoryResultInMemory() {

		Load shortLoad = new LoadBuilder(loadScenario1).amountOfThreads(10).stopDecision(duration(2 * SECOND))
				.throttle(2, PER_SECOND, PER_THREAD).build();

		/*
		 * After this execution the Result are read from memory and used in the
		 * ResultChart
		 */
		Result result = new ExecutionBuilder(shortLoad).storeResultRuntime().build().execute().andWait()
				.getReportedResultFromResultFile();

		ResultChart resultChart = new ResultChart(result);
		resultChart.waitUntilClosed();

		result.summaryStandard().build().prettyPrint();
	}
}