package loadtests.examples.loadcoder.bestpractise;

import static com.loadcoder.statics.LogbackLogging.getNewLogDir;
import static com.loadcoder.statics.LogbackLogging.setResultDestination;
import static com.loadcoder.statics.Statics.SECOND;
import static com.loadcoder.statics.Statics.duration;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import com.loadcoder.load.chart.logic.RuntimeChart;
import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenarioTyped;
import com.loadcoder.result.Result;

/**
 * This and the other classes inside the package bestpractise are examplifying the best practise
 * approach of how to structure your Loadcoder loadtest code.
 */
public class LoadTest {

	public static void main(String[] args) {
		new LoadTest().loadtest();
	}
	
	@Test
	public void testScenario() {
		Result result = new LoadTest().getScenario().test();
		assertEquals(0, result.getAmountOfFails());
		assertTrue(result.getAmountOfTransactions() != 0);
	}
	
	LoadScenarioTyped<ThreadInstance> loadScenario = new LoadScenarioTyped<ThreadInstance>() {

		@Override
		public void loadScenario(ThreadInstance t) {
			/*
			 * Instead of implementing one giant method, the load test logic is broken out
			 * into separate classes that each can be tested separately, see the test
			 * testGreetingsLogicAlone
			 */
			t.getGreetingsLogic().performGreeting();
			t.getCustomerLogic().getCustomerAge();
		}

		@Override
		public ThreadInstance createInstance() {
			/*
			 * The ThreadInstance created here for each thread will be a container for
			 * everything uniquely to each specific thead, and will in this case also hold
			 * separate instances for the load test logic.
			 */
			return new ThreadInstance(this);
		}
	};
	
	public LoadScenarioTyped<ThreadInstance> getScenario(){
		return loadScenario;
	}
	
	/**
	 * This test exemplifies a best practice of how to structure a loadcoder test
	 * 
	 * This test requires that the rest service is already started and available at
	 * 7060 To start the spring boot server, execute the following command inside
	 * the directory loadcoder-examples/server: mvn spring-boot:run
	 */
	public void loadtest() {

		// Giving the test a unique directory for the result logs
		setResultDestination(getNewLogDir("target", "bestPracticeLoadTest"));

		Load load = new LoadBuilder(getScenario()).amountOfThreads(5).rampup(10 * SECOND)
				.stopDecision(duration(30 * SECOND)).build();

		RuntimeChart runtimeChart = new RuntimeChart();

		Result result = new ExecutionBuilder(load).storeAndConsumeResultRuntime(runtimeChart).build().execute()
				.andWait().getReportedResultFromResultFile();

		result.summaryStandard().build().prettyPrint();
		
		runtimeChart.waitUntilClosed();

	}
	

}