package loadtests.examples.spring;

import java.security.cert.X509Certificate;

import javax.net.ssl.SSLContext;

import org.apache.http.client.HttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.util.ResourceUtils;
import org.springframework.web.client.RestTemplate;

/**
 * Example class for how to use Spring library to make http calls
 */
public class SpringExamples {

	RestTemplate defaultClient = new RestTemplate();
	RestTemplate acceptAllTLS = getTLSClient();
	RestTemplate acceptAllMTLS = getMTLSClient();
	
	public static void main(String[] args) {
		SpringExamples example = new SpringExamples();
		example.runExample();
	}

	private void runExample() {
		makeCall("https://localhost:8490/customer/get?email=hello", acceptAllMTLS);
	}

	public void makeCall(String url, RestTemplate t) {
		try {
			HttpEntity<String> req = new HttpEntity<>("");
			ResponseEntity<String>resp = t.exchange(url, HttpMethod.GET, req, String.class);
			System.out.println(resp.getBody());
		}catch(Exception e) {
			System.out.println(e);
			throw new RuntimeException(e);
		}
	}
	
	
	private RestTemplate getTLSClient() {
		try {
			return createRestTemplate(createSSLContextBuilder().build());
		}catch(Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	private RestTemplate getMTLSClient() {
		try {
			
			final String allPassword = "changeit";
			SSLContext context = createSSLContextBuilder()
					.loadKeyMaterial(ResourceUtils.getFile("classpath:clientkeystore.jks"),
					 allPassword.toCharArray(), allPassword.toCharArray()).build();

			return createRestTemplate(context);
		}catch(Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	private SSLContextBuilder createSSLContextBuilder(){
		try {
			final String allPassword = "changeit";
			 TrustStrategy acceptingTrustStrategy = (X509Certificate[] chain, String authType) -> {
				 return true;};
				 SSLContextBuilder builder = SSLContextBuilder
			 .create()
			 .loadKeyMaterial(ResourceUtils.getFile("classpath:clientkeystore.jks"),
			 allPassword.toCharArray(), allPassword.toCharArray())
			 
			 .loadTrustMaterial(null, acceptingTrustStrategy);
			 return builder;
		}catch(Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	RestTemplate createRestTemplate(SSLContext sslContext){
		HttpClient client = HttpClients.custom()
				 .setSSLContext(sslContext)
				 .build();
				
				HttpComponentsClientHttpRequestFactory requestFactory =
				 new HttpComponentsClientHttpRequestFactory();
				requestFactory.setHttpClient(client);
				
				RestTemplate restTemplate = new RestTemplate(requestFactory);
				return restTemplate;
	}
}
