package loadtests;

import static com.loadcoder.statics.Statics.*;
import static loadtests.tools.ClientUtils.createURLWithPort;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.HttpStatus;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.RestTemplate;

import com.loadcoder.load.scenario.ExecutionBuilder;
import com.loadcoder.load.scenario.Load;
import com.loadcoder.load.scenario.LoadBuilder;
import com.loadcoder.load.scenario.LoadScenarioTyped;
import com.loadcoder.result.Result;

import server.MyApp;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = MyApp.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class SpringBootLoadTests {

	@LocalServerPort
	private int port;

	RestTemplate restTemplate = new RestTemplate();

	/**
	 * This test will start spring boot instance in runtime and execute a load test towards it
	 */
	@Test
	public void loadtestRuntimeSpringBootService() {

		LoadScenarioTyped<RestTemplate> loadScenario = new LoadScenarioTyped<RestTemplate>() {

			@Override
			public void loadScenario(RestTemplate t) {

				load("greeting", () -> {
					return t.getForEntity(createURLWithPort("/api/greeting?name=Loadcoder", port), String.class);
				}).handleResult(resultHandler -> {
					if (!resultHandler.getResponse().getStatusCode().equals(HttpStatus.OK)) {
						resultHandler.setStatus(false);
					}
				}).perform();
			}

			@Override
			public RestTemplate createInstance() {
				return new RestTemplate();
			}
		};

		Load load = new LoadBuilder(loadScenario).amountOfThreads(5).rampup(10 * SECOND)
				.stopDecision(duration(3 * SECOND)).build();

		Result result = new ExecutionBuilder(load).storeResultRuntime().build().execute().andWait()
				.getResultFromMemory();

		result.summaryStandard().build().prettyPrint();
		
		
	}

}